from typing import TYPE_CHECKING, Optional

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client, Best, SearchResult


@model
class Search(YandexMusicObject):
    """Класс, представляющий результаты поиска.

    Attributes:
        search_request_id (:obj:`str`): ID запроса.
        text (:obj:`str`): Текст запроса.
        best (:obj:`yandex_music.Best`, optional): Лучший результат.
        albums (:obj:`yandex_music.SearchResult`, optional): Найденные альбомы.
        artists (:obj:`yandex_music.SearchResult`, optional): Найденные исполнители.
        playlists (:obj:`yandex_music.SearchResult`, optional): Найденные плейлисты.
        tracks (:obj:`yandex_music.SearchResult`, optional): Найденные треки.
        videos (:obj:`yandex_music.SearchResult`, optional): Найденные видео.
        users (:obj:`yandex_music.SearchResult`, optional): Найденные пользователи.
        podcasts (:obj:`yandex_music.SearchResult`, optional): Найденные подкасты.
        podcast_episodes (:obj:`yandex_music.SearchResult`, optional): Найденные выпуски подкастов.
        type (:obj:`str`), optional: Тип результата по которому искали (аргумент в Client.search).
        page (:obj:`int`, optional): Текущая страница.
        per_page (:obj:`int`, optional): Результатов на странице.
        misspell_result (:obj:`str`, optional): Запрос с автоматическим исправлением.
        misspell_original (:obj:`str`, optional): Оригинальный запрос.
        misspell_corrected (:obj:`bool`, optional): Был ли исправлен запрос.
        nocorrect (:obj:`bool`, optional): Было ли отключено исправление результата.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    search_request_id: str
    text: str
    best: Optional['Best']
    albums: Optional['SearchResult']
    artists: Optional['SearchResult']
    playlists: Optional['SearchResult']
    tracks: Optional['SearchResult']
    videos: Optional['SearchResult']
    users: Optional['SearchResult']
    podcasts: Optional['SearchResult']
    podcast_episodes: Optional['SearchResult']
    type: Optional[str] = None
    page: Optional[int] = None
    per_page: Optional[int] = None
    misspell_result: Optional[str] = None
    misspell_original: Optional[str] = None
    misspell_corrected: Optional[bool] = None
    nocorrect: Optional[bool] = None
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (
            self.search_request_id,
            self.text,
            self.best,
            self.albums,
            self.artists,
            self.playlists,
            self.tracks,
            self.videos,
            self.users,
            self.podcasts,
            self.podcast_episodes,
        )

    def get_page(self, page: int, *args, **kwargs) -> Optional['Search']:
        """Получение определённой страницы поиска.

        Args:
            page (:obj:`int`): Номер страницы.

        Returns:
            :obj:`yandex_music.Search` | :obj:`None`: Страница результата поиска или :obj:`None`.
        """
        return self.client.search(self.text, self.nocorrect, self.type_, page, *args, **kwargs)

    async def get_page_async(self, page: int, *args, **kwargs) -> Optional['Search']:
        """Получение определённой страницы поиска.

        Args:
            page (:obj:`int`): Номер страницы.

        Returns:
            :obj:`yandex_music.Search` | :obj:`None`: Страница результата поиска или :obj:`None`.
        """
        return await self.client.search(self.text, self.nocorrect, self.type_, page, *args, **kwargs)

    def next_page(self, *args, **kwargs) -> Optional['Search']:
        """Получение следующей страницы поиска.

        Returns:
            :obj:`yandex_music.Search` | :obj:`None`: Следующая страница результата поиска или :obj:`None`.
        """
        return self.get_page(self.page + 1, *args, **kwargs)

    async def next_page_async(self, *args, **kwargs) -> Optional['Search']:
        """Получение следующей страницы поиска.

        Returns:
            :obj:`yandex_music.Search` | :obj:`None`: Следующая страница результата поиска или :obj:`None`.
        """
        return await self.get_page_async(self.page + 1, *args, **kwargs)

    def prev_page(self, *args, **kwargs) -> Optional['Search']:
        """Получение предыдущей страницы поиска.

        Returns:
            :obj:`yandex_music.Search` | :obj:`None`: Предыдущая страница результата поиска или :obj:`None`.
        """
        return self.get_page(self.page - 1, *args, **kwargs)

    async def prev_page_async(self, *args, **kwargs) -> Optional['Search']:
        """Получение предыдущей страницы поиска.

        Returns:
            :obj:`yandex_music.Search` | :obj:`None`: Предыдущая страница результата поиска или :obj:`None`.
        """
        return await self.get_page_async(self.page - 1, *args, **kwargs)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['Search']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.Search`: Результаты поиска.
        """
        if not data:
            return None

        data = super(Search, cls).de_json(data, client)
        from yandex_music import SearchResult, Best

        # в ОЧЕНЬ редких случаях сервер творит дичь и может вернуть результат плейлистов в поле artists
        # или вернуть в поле users результаты с плейлистами

        # очень редких это около 10 запросов за 3 месяца работы стороннего клиента

        data['best'] = Best.de_json(data.get('best'), client)
        data['albums'] = SearchResult.de_json(data.get('albums'), client, 'album')
        data['artists'] = SearchResult.de_json(data.get('artists'), client, 'artist')
        data['playlists'] = SearchResult.de_json(data.get('playlists'), client, 'playlist')
        data['tracks'] = SearchResult.de_json(data.get('tracks'), client, 'track')
        data['videos'] = SearchResult.de_json(data.get('videos'), client, 'video')
        data['users'] = SearchResult.de_json(data.get('users'), client, 'user')
        data['podcasts'] = SearchResult.de_json(data.get('podcasts'), client, 'podcast')
        data['podcast_episodes'] = SearchResult.de_json(data.get('podcast_episodes'), client, 'podcast_episode')

        return cls(client=client, **data)

    # camelCase псевдонимы

    #: Псевдоним для :attr:`get_page`
    getPage = get_page
    #: Псевдоним для :attr:`get_page_async`
    getPageAsync = get_page_async
    #: Псевдоним для :attr:`next_page`
    nextPage = next_page
    #: Псевдоним для :attr:`next_page_async`
    nextPageASync = next_page_async
    #: Псевдоним для :attr:`prev_page`
    prevPage = prev_page
    #: Псевдоним для :attr:`prev_page_async`
    prevPageAsync = prev_page_async



from typing import TYPE_CHECKING, Optional, List

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client, Track


@model
class TrackWithAds(YandexMusicObject):
    """Класс, представляющий трек с рекламой.

    Note:
        Поле `type` встречалось только с значением `track`.

    Attributes:
        type (:obj:`str`): Тип TODO.
        track (:obj:`yandex_music.Track`): Трек.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    type: str
    track: Optional['Track']
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.type, self.track)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['TrackWithAds']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.TrackWithAds`: Трек с рекламой.
        """
        if not data:
            return None

        data = super(TrackWithAds, cls).de_json(data, client)
        from yandex_music import Track

        data['track'] = Track.de_json(data.get('track'), client)

        return cls(client=client, **data)

    @classmethod
    def de_list(cls, data: dict, client: 'Client') -> List['TrackWithAds']:
        """Десериализация списка объектов.

        Args:
            data (:obj:`list`): Список словарей с полями и значениями десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`list` из :obj:`yandex_music.TrackWithAds`: Треки с рекламой.
        """
        if not data:
            return []

        tracks_with_ads = list()
        for track_with_ads in data:
            tracks_with_ads.append(cls.de_json(track_with_ads, client))

        return tracks_with_ads



from typing import TYPE_CHECKING, Optional

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client, Value


@model
class DiscreteScale(YandexMusicObject):
    """Класс, представляющий дискретное значение.

    Note:
        Известные значения поля `type`: `discrete-scale`.

    Attributes:
        type (:obj:`str`): Тип.
        name (:obj:`str`): Название.
        min (:obj:`yandex_music.Value`): Минимальное значение.
        max (:obj:`yandex_music.Value`): Максимальное значение.
        client (:obj:`yandex_music.Client` optional): Клиент Yandex Music.
    """

    type: str
    name: str
    min: Optional['Value']
    max: Optional['Value']
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.type, self.name, self.min, self.max)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['DiscreteScale']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.DiscreteScale`: Дискретное значение.
        """
        if not data:
            return None

        data = super(DiscreteScale, cls).de_json(data, client)
        from yandex_music import Value

        data['min'] = Value.de_json(data.get('min'), client)
        data['max'] = Value.de_json(data.get('max'), client)

        return cls(client=client, **data)



from typing import TYPE_CHECKING, Optional

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client, User, CaseForms


@model
class MadeFor(YandexMusicObject):
    """Класс, представляющий пользователя, для которого был сделан плейлист.

    Attributes:
        user_info (:obj:`yandex_music.User`): Пользователь, для которого был сделан плейлист.
        case_forms (:obj:`yandex_music.CaseForms`): Склонение имени пользователя, для которого был сделан плейлист.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    user_info: Optional['User']
    case_forms: Optional['CaseForms']
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.user_info, self.case_forms)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['MadeFor']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.MadeFor`: Пользователь, для которого был сделан плейлист.
        """
        if not data:
            return None

        data = super(MadeFor, cls).de_json(data, client)
        from yandex_music import User, CaseForms

        data['user_info'] = User.de_json(data.get('user_info'), client)
        data['case_forms'] = CaseForms.de_json(data.get('case_forms'), client)

        return cls(client=client, **data)



from typing import TYPE_CHECKING, Optional, List

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client, Artist, Track


@model
class ArtistEvent(YandexMusicObject):
    """Класс, представляющий артиста в событии фида.

    Attributes:
        artist (:obj:`yandex_music.Artist` | :obj:`None`): Артист.
        tracks (:obj:`list` :obj:`yandex_music.Track`): Треки.
        similar_to_artists_from_history (:obj:`list` :obj:`yandex_music.Artist`): Похожие артисты из истории.
        subscribed (:obj:`bool`): Подписан ли на событие.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    artist: Optional['Artist']
    tracks: List['Track']
    similar_to_artists_from_history: List['Artist']
    subscribed: Optional['bool'] = None
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.artist, self.tracks, self.similar_to_artists_from_history)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['ArtistEvent']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.ArtistEvent`: Артист из события фида.
        """
        if not data:
            return None

        data = super(ArtistEvent, cls).de_json(data, client)
        from yandex_music import Artist, Track

        data['artist'] = Artist.de_json(data.get('artist'), client)
        data['tracks'] = Track.de_list(data.get('tracks'), client)
        data['similar_to_artists_from_history'] = Artist.de_list(data.get('similar_to_artists_from_history'), client)

        return cls(client=client, **data)

    @classmethod
    def de_list(cls, data: dict, client: 'Client') -> List['ArtistEvent']:
        """Десериализация списка объектов.

        Args:
            data (:obj:`list`): Список словарей с полями и значениями десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`list` из :obj:`yandex_music.ArtistEvent`: Артисты из события фида.
        """
        if not data:
            return []

        artist_events = list()
        for artist_event in data:
            artist_events.append(cls.de_json(artist_event, client))

        return artist_events



from typing import TYPE_CHECKING, Optional, List

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client, Track, Chart


@model
class ChartItem(YandexMusicObject):
    """Класс, представляющий трек в чарте.

    Attributes:
        track (:obj:`yandex_music.Track` | :obj:`None`): Трек.
        chart (:obj:`yandex_music.Chart` | :obj:`None`): Элемент чарт.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    track: Optional['Track']
    chart: Optional['Chart']
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.track, self.chart)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['ChartItem']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.ChartItem`: Трек в чарте.
        """
        if not data:
            return None

        data = super(ChartItem, cls).de_json(data, client)
        from yandex_music import Chart, Track

        data['track'] = Track.de_json(data.get('track'), client)
        data['chart'] = Chart.de_json(data.get('chart'), client)

        return cls(client=client, **data)

    @classmethod
    def de_list(cls, data: dict, client: 'Client') -> List['ChartItem']:
        """Десериализация списка объектов.

        Args:
            data (:obj:`list`): Список словарей с полями и значениями десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`list` из :obj:`yandex_music.ChartItem`: Треки в чартах.
        """
        if not data:
            return []

        tracks = list()
        for track in data:
            tracks.append(cls.de_json(track, client))

        return tracks



from typing import TYPE_CHECKING, Optional

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client


@model
class MetaData(YandexMusicObject):
    """Класс, представляющий метаданные трека.

    Attributes:
        album (:obj:`str`, optional): Название альбома.
        volume (:obj:`int`, optional): Диск (раздел).
        year (:obj:`int`, optional): Год выхода.
        number (:obj:`int`, optional): Позиция в альбоме.
        genre (:obj:`str`, optional): Жанр.
        lyricist (:obj:`str`, optional): Текст песни. Есть только у пользовательских треков.
        version (:obj:`str`, optional): Версия TODO.
        composer (:obj:`str`, optional): Композитор TODO.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    album: Optional[str] = None
    volume: Optional[int] = None
    year: Optional[int] = None
    number: Optional[int] = None
    genre: Optional[str] = None
    lyricist: Optional[str] = None
    version: Optional[str] = None
    composer: Optional[str] = None
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.album, self.volume, self.year)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['MetaData']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.MetaData`: Метаданные трека
        """
        if not data:
            return None

        data = super(MetaData, cls).de_json(data, client)

        return cls(client=client, **data)



from typing import TYPE_CHECKING, Optional, List

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client, Value


@model
class Enum(YandexMusicObject):
    """Класс, представляющий перечисления.

    Attributes:
        type (:obj:`str`): Тип перечисления.
        name (:obj:`str`): Название перечисления.
        possible_values (:obj:`list` из :obj:`yandex_Music.Value`): Доступные значения.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
        **kwargs: Произвольные ключевые аргументы полученные от API.
    """

    type: str
    name: str
    possible_values: List['Value']
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.type, self.name, self.possible_values)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['Enum']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.Enum`: Перечисление.
        """
        if not data:
            return None

        data = super(Enum, cls).de_json(data, client)
        from yandex_music import Value

        data['possible_values'] = Value.de_list(data.get('possible_values'), client)

        return cls(client=client, **data)



from typing import TYPE_CHECKING, Optional

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client, Account, Permissions, Subscription, Plus, StationData, Alert


@model
class Status(YandexMusicObject):
    """Класс, представляющий подробную информацию об аккаунте пользователя.

    Attributes:
        account (:obj:`yandex_music.Account`): Основная информация об аккаунте
        permissions (:obj:`yandex_music.Permissions`): Информация о правах пользователя.
        advertisement (:obj:`str`, optional): Рекламное объявление.
        subscription (:obj:`yandex_music.Subscription`, optional): Информация о подписках.
        cache_limit (:obj:`int`, optional): Максимальное количество загруженных треков.
        subeditor (:obj:`bool`, optional): Наличие статуса модератора проверки корректности информации.
        subeditor_level (:obj:`int`, optional): Уровень статуса модератора.
        plus (:obj:`yandex_music.Plus`, optional): Информация о Plus подписке.
        default_email (:obj:`str`, optional): Основной e-mail адрес аккаунта.
        skips_per_hour (:obj:`int`, optional): Количество переключение треков на радио в час.
        station_exists (:obj:`bool`, optional): Наличие личной станции.
        station_data (:obj:`yandex_music.StationData`, optional): Информация о личной станции.
        bar_below (:obj:`yandex_music.Alert`, optional): Блок с предупреждениями о конце подписке и подарках.
        premium_region (:obj:`int`, optional): Регион TODO.
        experiment (:obj:`int`, optional): Включенная новая фича на аккаунте (её ID) TODO.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    account: Optional['Account']
    permissions: Optional['Permissions']
    advertisement: Optional[str] = None
    subscription: Optional['Subscription'] = None
    cache_limit: Optional[int] = None
    subeditor: Optional[bool] = None
    subeditor_level: Optional[int] = None
    plus: Optional['Plus'] = None
    default_email: Optional[str] = None
    skips_per_hour: Optional[int] = None
    station_exists: Optional[bool] = None
    station_data: Optional['StationData'] = None
    bar_below: Optional['Alert'] = None
    premium_region: Optional[int] = None
    experiment: Optional[int] = None
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.account, self.permissions)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['Status']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.Status`: Информация об аккаунте пользователя.
        """
        if not data:
            return None

        data = super(Status, cls).de_json(data, client)
        from yandex_music import Account, Permissions, Plus, Subscription, StationData, Alert

        data['account'] = Account.de_json(data.get('account'), client)
        data['permissions'] = Permissions.de_json(data.get('permissions'), client)
        data['subscription'] = Subscription.de_json(data.get('subscription'), client)
        data['plus'] = Plus.de_json(data.get('plus'), client)
        data['station_data'] = StationData.de_json(data.get('station_data'), client)
        data['bar_below'] = Alert.de_json(data.get('bar_below'), client)

        return cls(client=client, **data)



from typing import TYPE_CHECKING, Optional, List

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client


@model
class Vinyl(YandexMusicObject):
    """Класс, представляющий виниловую пластинку.

    Attributes:
        url (:obj:`str`): Ссылка на пластинку в магазине.
        title (:obj:`str`): Заголовок.
        year (:obj:`int`): Год выпуска.
        price (:obj:`int`): Цена.
        media (:obj:`str`): Средство распространения.
        offer_id (:obj:`int`): Уникальный идентификатор предложения.
        artist_ids (:obj:`list` из :obj:`int`): Перечень уникальный идентификаторов исполнителей.
        picture (:obj:`str`, optional): Ссылка на обложку.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
        **kwargs: Произвольные ключевые аргументы полученные от API.
    """

    url: str
    title: str
    year: int
    price: int
    media: str
    offer_id: int
    artist_ids: List[int]
    picture: Optional[str] = None
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (
            self.title,
            self.price,
            self.year,
            self.url,
            self.price,
            self.media,
            self.offer_id,
            self.artist_ids,
        )

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['Vinyl']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.Vinyl`: Ваниловая пластинка.
        """
        if not data:
            return None

        data = super(Vinyl, cls).de_json(data, client)

        return cls(client=client, **data)

    @classmethod
    def de_list(cls, data: dict, client: 'Client') -> List['Vinyl']:
        """Десериализация списка объектов.

        Args:
            data (:obj:`list`): Список словарей с полями и значениями десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`list` из :obj:`yandex_music.Vinyl`: Ваниловые пластинки.
        """
        if not data:
            return []

        vinyls = list()
        for vinyl in data:
            vinyls.append(cls.de_json(vinyl, client))

        return vinyls



import dataclasses
import logging
import keyword
from abc import ABCMeta
from typing import TYPE_CHECKING, Optional

if TYPE_CHECKING:
    from yandex_music import Client

ujson: bool = False
try:
    import ujson as json

    ujson = True
except ImportError:
    import json

reserved_names = keyword.kwlist

logger = logging.getLogger(__name__)
new_issue_by_template_url = 'https://bit.ly/3dsFxyH'


class YandexMusicObject:
    __metaclass__ = ABCMeta
    _id_attrs: tuple = ()

    def __str__(self) -> str:
        return str(self.to_dict())

    def __repr__(self) -> str:
        return str(self)

    def __getitem__(self, item):
        return self.__dict__[item]

    @staticmethod
    def report_unknown_fields_callback(obj, unknown_fields):
        logger.warning(
            f'Found unknown fields received from API! Please copy warn message '
            f'and send to {new_issue_by_template_url} (github issue), thank you!'
        )
        logger.warning(f'Type: {type(obj)}; fields: {unknown_fields}')

    @classmethod
    def de_json(cls, data: dict, client: Optional['Client']) -> Optional[dict]:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.YandexMusicObject` | :obj:`None`: :obj:`yandex_music.YandexMusicObject` или :obj:`None`.
        """
        if not data:
            return None

        data = data.copy()

        fields = {f.name for f in dataclasses.fields(cls)}

        cleaned_data = dict()
        unknown_data = dict()

        for k, v in data.items():
            if k in fields:
                cleaned_data[k] = v
            else:
                unknown_data[k] = v

        if client.report_unknown_fields and unknown_data:
            YandexMusicObject.report_unknown_fields_callback(cls, unknown_data)

        return cleaned_data

    def to_json(self, for_request=False) -> str:
        """Сериализация объекта.

        Args:
            for_request (:obj:`bool`): Подготовить ли объект для отправки в теле запроса.

        Returns:
            :obj:`str`: Сериализованный в JSON объект.
        """
        return json.dumps(self.to_dict(for_request), ensure_ascii=not ujson)

    def to_dict(self, for_request=False) -> dict:
        """Рекурсивная сериализация объекта.

        Args:
            for_request (:obj:`bool`): Перевести ли обратно все поля в camelCase и игнорировать зарезервированные слова.

        Note:
            Исключает из сериализации `client` и `_id_attrs` необходимые в `__eq__`.

            К зарезервированным словам добавляет "_" в конец.

        Returns:
            :obj:`dict`: Сериализованный в dict объект.
        """

        def parse(val):
            if hasattr(val, 'to_dict'):
                return val.to_dict(for_request)
            elif isinstance(val, list):
                return [parse(it) for it in val]
            elif isinstance(val, dict):
                return {key: parse(value) for key, value in val.items()}
            else:
                return val

        data = self.__dict__.copy()
        data.pop('client', None)
        data.pop('_id_attrs', None)

        if for_request:
            for k, v in data.copy().items():
                camel_case = ''.join(word.title() for word in k.split('_'))
                camel_case = camel_case[0].lower() + camel_case[1:]

                data.pop(k)
                data.update({camel_case: v})
        else:
            for k, v in data.copy().items():
                if k.lower() in reserved_names:
                    data.pop(k)
                    data.update({f'{k}_': v})

        return parse(data)

    def __eq__(self, other) -> bool:
        """Проверка на равенство двух объектов.

        Note:
            Проверка осуществляется по определённым атрибутам классов, перечисленных в множестве `_id_attrs`.

        Returns:
            :obj:`bool`: Одинаковые ли объекты (по содержимому).
        """
        if isinstance(other, self.__class__):
            return self._id_attrs == other._id_attrs
        return super(YandexMusicObject, self).__eq__(other)

    def __hash__(self) -> int:
        """Реализация хеш-функции на основе ключевых атрибутов.

        Note:
            Так как перечень ключевых атрибутов хранится в виде множества, для вычисления хеша он замораживается.

        Returns:
            :obj:`int`: Хеш объекта.
        """
        if self._id_attrs:
            frozen_attrs = tuple(frozenset(attr) if isinstance(attr, list) else attr for attr in self._id_attrs)
            return hash((self.__class__, frozen_attrs))
        return super(YandexMusicObject, self).__hash__()



from typing import List, TYPE_CHECKING, Optional

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client, Context, Queue


@model
class QueueItem(YandexMusicObject):
    """Класс, представляющий очередь треков в списке очередей устройств.

    Attributes:
        id (:obj:`str`): Уникальный идентификатор очереди.
        context (:obj:`yandex_music.Context` | :obj:`None`): Объект по которому построена очередь.
        modified (:obj:`str`): Дата последнего изменения.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    id: str
    context: Optional['Context']
    modified: str
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.id,)

    def fetch_queue(self, *args, **kwargs) -> Optional['Queue']:
        """Сокращение для::

        client.queue(id, *args, **kwargs)
        """
        return self.client.queue(self.id, *args, **kwargs)

    async def fetch_queue_async(self, *args, **kwargs) -> Optional['Queue']:
        """Сокращение для::

        await client.queue(id, *args, **kwargs)
        """
        return await self.client.queue(self.id, *args, **kwargs)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['QueueItem']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.QueueItem`: Очередь в списке.
        """
        if not data:
            return None

        from yandex_music import Context

        data = super(QueueItem, cls).de_json(data, client)
        data['context'] = Context.de_json(data.get('context'), client)

        return cls(client=client, **data)

    @classmethod
    def de_list(cls, data: dict, client: 'Client') -> List['QueueItem']:
        """Десериализация списка объектов.

        Args:
            data (:obj:`list`): Список словарей с полями и значениями десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`list` из :obj:`yandex_music.QueueItem`: Список очередей всех устройств.
        """
        if not data:
            return []

        return [cls.de_json(queue, client) for queue in data]

    # camelCase псевдонимы

    #: Псевдоним для :attr:`fetch_queue`
    fetchQueue = fetch_queue
    #: Псевдоним для :attr:`fetch_queue_async`
    fetchQueueAsync = fetch_queue_async



from typing import Any, TYPE_CHECKING, Optional, List

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import (
        Client,
        User,
        Cover,
        MadeFor,
        TrackShort,
        PlaylistAbsence,
        PlayCounter,
        PlaylistRecommendations,
        Artist,
        TrackId,
        Contest,
        OpenGraphData,
        Brand,
    )


@model
class Playlist(YandexMusicObject):
    """Класс, представляющий плейлист.

    Note:
        Под полями с заглушками понимаются поля, которые доступны у умных плейлистов тогда, когда у сервиса мало
        данных для генерации плейлиста.

        Известные значения `visibility`: `public` - публичный плейлист, `private` - приватный плейлист.

        Известные значения `generated_playlist_type`: `playlistOfTheDay` - Плейлист дня, `recentTracks` - Премьера,
        `neverHeard` - Дежавю, `podcasts` - Подкасты недели, `missedLikes` - Тайник, `origin` - Плейлист с Алисой.

        Известные значения `type`: `missedLikes`, `recentTracks`.

    Attributes:
        owner (:obj:`yandex_music.User`, optional): Владелец плейлиста.
        cover (:obj:`yandex_music.Cover`, optional): Обложка альбома.
        made_for (:obj:`yandex_music.MadeFor`, optional): Пользователь для которого был создан плейлист. Присутствует
            только у персональных плейлистов.
        play_counter (:obj:`yandex_music.PlayCounter`, optional): Счётчик дней. Присутствует только у плейлиста дня.
        playlist_absence (:obj:`yandex_music.PlaylistAbsence`, optional): Причина отсутствия плейлиста.
        uid (:obj:`int`, optional): Идентификатор владельца плейлиста.
        kind (:obj:`int`, optional): Идентификатор плейлиста.
        title (:obj:`str`, optional): Название плейлиста.
        track_count (:obj:`int`, optional): Количество треков.
        tags (:obj:`list`, optional): Список тегов плейлиста.
        revision (:obj:`int`, optional): Актуальность данных TODO.
        snapshot (:obj:`int`, optional): Версия плейлиста. Увеличивается на 1 при каждом изменении.
        visibility (:obj:`str`, optional): Видимость плейлиста.
        collective (:obj:`bool`, optional): Есть ли у плейлиста соавторы.
        url_part (:obj:`str`, optional): Часть ссылки на плейлист ('daily`).
        created (:obj:`str`, optional): Дата создания в формате ISO 8601.
        modified (:obj:`str`, optional): Дата последнего изменения в формате ISO 8601.
        available (:obj:`bool`, optional): Доступен TODO.
        is_banner (:obj:`bool`, optional): Является ли банером TODO.
        is_premiere (:obj:`bool`, optional): Является ли премьерой TODO.
        duration_ms (:obj:`int`, optional): Длительность в миллисекундах.
        og_image (:obj:`str`, optional): Ссылка на превью Open Graph.
        og_title (:obj:`str`, optional): Заголовок Open Graph.
        og_description (:obj:`str`, optional): Описание Open Graph.
        image (:obj:`str`, optional): Изображение TODO.
        cover_without_text (:obj:`yandex_music.Cover`, optional): Обложка без текста.
        contest (:obj:`yandex_music.Contest`, optional): Контест TODO.
        background_color (:obj:`str`, optional): Цвет заднего фона TODO.
        text_color (:obj:`str`, optional): Цвет текста TODO.
        id_for_from (:obj:`str`, optional): Откуда пришло событие (уникальный идентификатор объекта) TODO.
        dummy_description (:obj:`str`, optional): Описание-заглушка плейлиста.
        dummy_page_description (:obj:`str`, optional): Описание-заглушка страницы.
        dummy_cover (:obj:`str`, optional): Обложка-заглушка.
        dummy_rollover_cover (:obj:`str`, optional): Обложка-заглушка TODO.
        og_data (:obj:`yandex_music.OpenGraphData`, optional): Данные для OpenGraph.
        branding (:obj:`yandex_music.Brand`): Бренд.
        metrika_id (:obj:`int`, optional): Уникальный идентификатор счётчика на Яндекс.Метрика.
        coauthors (:obj:`list` из :obj:`int`, optional): Перечень ID аккаунтов соавторов плейлиста.
        top_artist (:obj:`list` из :obj:`yandex_music.Artist`, optional): Топ артистов TODO.
        recent_tracks (:obj:`list` из :obj:`yandex_music.TrackId`, optional): Список ID недавних треков.
        tracks (:obj:`list` из :obj:`yandex_music.TrackShort`, optional): Список треков.
        prerolls (:obj:`list`, optional): Прерол, проигрываемый перед плейлистом. Присутствует только у персональных
            плейлистов.
        likes_count (:obj:`int`, optional): Количество лайков.
        similar_playlists (:obj:`list` из :obj:`yandex_music.Playlist`, optional): Похожие плейлисты.
        last_owner_playlists (:obj:`list` из :obj:`yandex_music.Playlist`, optional): Последние плейлисты владельца.
        generated_playlist_type (:obj:`str`, optional): Тип генерируемого плейлиста.
        animated_cover_uri (:obj:`str`, optional): Ссылка на анимированную обложку.
        ever_played (:obj:`str`, optional): Играл ли этот плейлист. Присутствует только у персональных плейлистов. TODO
        description (:obj:`str`, optional): Описание плейлиста с разметкой Markdown.
        description_formatted (:obj:`str`, optional): Описание плейлиста. Только текст, без разметки.
        playlist_uuid (:obj:`str`, optional): TODO.
        type (:obj:`str`, optional): TODO.
        ready (:obj:`bool`, optional): Готовность TODO.
        is_for_from: TODO.
        regions: Регион TODO.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    owner: Optional['User']
    cover: Optional['Cover']
    made_for: Optional['MadeFor']
    play_counter: Optional['PlayCounter']
    playlist_absence: Optional['PlaylistAbsence']
    uid: Optional[int] = None
    kind: Optional[int] = None
    title: Optional[str] = None
    track_count: Optional[int] = None
    tags: Optional[list] = None
    revision: Optional[int] = None
    snapshot: Optional[int] = None
    visibility: Optional[str] = None
    collective: Optional[bool] = None
    url_part: Optional[str] = None
    created: Optional[str] = None
    modified: Optional[str] = None
    available: Optional[bool] = None
    is_banner: Optional[bool] = None
    is_premiere: Optional[bool] = None
    duration_ms: Optional[int] = None
    og_image: Optional[str] = None
    og_title: Optional[str] = None
    og_description: Optional[str] = None
    image: Optional[str] = None
    cover_without_text: Optional['Cover'] = None
    contest: Optional['Contest'] = None
    background_color: Optional[str] = None
    text_color: Optional[str] = None
    id_for_from: Optional[str] = None
    dummy_description: Optional[str] = None
    dummy_page_description: Optional[str] = None
    dummy_cover: Optional['Cover'] = None
    dummy_rollover_cover: Optional['Cover'] = None
    og_data: Optional['OpenGraphData'] = None
    branding: Optional['Brand'] = None
    metrika_id: Optional[int] = None
    coauthors: List[int] = None
    top_artist: List['Artist'] = None
    recent_tracks: List['TrackId'] = None
    tracks: List['TrackShort'] = None
    prerolls: Optional[list] = None
    likes_count: Optional[int] = None
    similar_playlists: List['Playlist'] = None
    last_owner_playlists: List['Playlist'] = None
    generated_playlist_type: Optional[str] = None
    animated_cover_uri: Optional[str] = None
    ever_played: Optional[bool] = None
    description: Optional[str] = None
    description_formatted: Optional[str] = None
    playlist_uuid: Optional[str] = None
    type: Optional[str] = None
    ready: Optional[bool] = None
    is_for_from: Any = None
    regions: Any = None
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.uid, self.kind, self.title, self.playlist_absence)

    @property
    def is_mine(self) -> bool:
        return self.owner.uid == self.client.me.account.uid

    @property
    def playlist_id(self) -> str:
        return f'{self.owner.uid}:{self.kind}'

    def get_recommendations(self, *args, **kwargs) -> Optional['PlaylistRecommendations']:
        """Сокращение для::

        client.users_playlists_recommendations(playlist.kind, playlist.owner.uid, *args, **kwargs)
        """
        return self.client.users_playlists_recommendations(self.kind, self.owner.uid, *args, **kwargs)

    async def get_recommendations_async(self, *args, **kwargs) -> Optional['PlaylistRecommendations']:
        """Сокращение для::

        await client.users_playlists_recommendations(playlist.kind, playlist.owner.uid, *args, **kwargs)
        """
        return await self.client.users_playlists_recommendations(self.kind, self.owner.uid, *args, **kwargs)

    def download_animated_cover(self, filename: str, size: str = '200x200') -> None:
        """Загрузка анимированной обложки.

        Args:
            filename (:obj:`str`): Путь для сохранения файла с названием и расширением (GIF).
            size (:obj:`str`, optional): Размер анимированной обложки.
        """
        self.client.request.download(f'https://{self.animated_cover_uri.replace("%%", size)}', filename)

    async def download_animated_cover_async(self, filename: str, size: str = '200x200') -> None:
        """Загрузка анимированной обложки.

        Args:
            filename (:obj:`str`): Путь для сохранения файла с названием и расширением (GIF).
            size (:obj:`str`, optional): Размер анимированной обложки.
        """
        await self.client.request.download(f'https://{self.animated_cover_uri.replace("%%", size)}', filename)

    def download_og_image(self, filename: str, size: str = '200x200') -> None:
        """Загрузка обложки.

        Используйте это только когда нет self.cover!

        Args:
            filename (:obj:`str`): Путь для сохранения файла с названием и расширением.
            size (:obj:`str`, optional): Размер обложки.
        """
        self.client.request.download(f'https://{self.og_image.replace("%%", size)}', filename)

    async def download_og_image_async(self, filename: str, size: str = '200x200') -> None:
        """Загрузка обложки.

        Используйте это только когда нет self.cover!

        Args:
            filename (:obj:`str`): Путь для сохранения файла с названием и расширением.
            size (:obj:`str`, optional): Размер обложки.
        """
        await self.client.request.download(f'https://{self.og_image.replace("%%", size)}', filename)

    def rename(self, name: str) -> None:
        client, kind = self.client, self.kind

        self.__dict__.clear()
        self.__dict__.update(client.users_playlists_name(kind, name).__dict__)

    async def rename_async(self, name: str) -> None:
        client, kind = self.client, self.kind

        self.__dict__.clear()
        self.__dict__.update((await client.users_playlists_name(kind, name)).__dict__)

    def like(self, *args, **kwargs) -> bool:
        """Сокращение для::

        client.users_likes_playlists_add(playlist.uid, user.id *args, **kwargs)
        """
        return self.client.users_likes_playlists_add(self.uid, self.client.me.account.uid, *args, **kwargs)

    async def like_async(self, *args, **kwargs) -> bool:
        """Сокращение для::

        await client.users_likes_playlists_add(playlist.uid, user.id *args, **kwargs)
        """
        return await self.client.users_likes_playlists_add(self.uid, self.client.me.account.uid, *args, **kwargs)

    def dislike(self, *args, **kwargs) -> bool:
        """Сокращение для::

        client.users_likes_playlists_remove(playlist.uid, user.id *args, **kwargs)
        """
        return self.client.users_likes_playlists_remove(self.uid, self.client.me.account.uid, *args, **kwargs)

    async def dislike_async(self, *args, **kwargs) -> bool:
        """Сокращение для::

        await client.users_likes_playlists_remove(playlist.uid, user.id *args, **kwargs)
        """
        return await self.client.users_likes_playlists_remove(self.uid, self.client.me.account.uid, *args, **kwargs)

    def fetch_tracks(self, *args, **kwargs) -> List['TrackShort']:
        """Сокращение для::

        client.users_playlists(playlist.kind, playlist.owner.id, *args, **kwargs).tracks
        """
        return self.client.users_playlists(self.kind, self.owner.uid, *args, **kwargs).tracks

    async def fetch_tracks_async(self, *args, **kwargs) -> List['TrackShort']:
        """Сокращение для::

        await client.users_playlists(playlist.kind, playlist.owner.id, *args, **kwargs).tracks
        """
        return await self.client.users_playlists(self.kind, self.owner.uid, *args, **kwargs).tracks

    def insert_track(self, track_id: int, album_id: int, *args, **kwargs) -> Optional['Playlist']:
        """Сокращение для::

        client.users_playlists_insert_track(self.kind, track_id, album_id, user_id=self.owner.uid,
        revision=self.revision, *args, **kwargs)
        """
        return self.client.users_playlists_insert_track(
            self.kind, track_id, album_id, user_id=self.owner.uid, revision=self.revision, *args, **kwargs
        )

    async def insert_track_async(self, track_id: int, album_id: int, *args, **kwargs) -> Optional['Playlist']:
        """Сокращение для::

        await client.users_playlists_insert_track(self.kind, track_id, album_id, user_id=self.owner.uid,
        revision=self.revision, *args, **kwargs)
        """
        return await self.client.users_playlists_insert_track(
            self.kind, track_id, album_id, user_id=self.owner.uid, revision=self.revision, *args, **kwargs
        )

    def delete_tracks(self, from_: int, to: int, *args, **kwargs) -> Optional['Playlist']:
        """Сокращение для::

        client.users_playlists_delete_track(self.kind, from_, to, self.revision, self.owner.uid, *args, **kwargs)
        """
        return self.client.users_playlists_delete_track(
            self.kind, from_, to, self.revision, self.owner.uid, *args, **kwargs
        )

    async def delete_tracks_async(self, from_: int, to: int, *args, **kwargs) -> Optional['Playlist']:
        """Сокращение для::

        await client.users_playlists_delete_track(self.kind, from_, to, self.revision, self.owner.uid, *args, **kwargs)
        """
        return await self.client.users_playlists_delete_track(
            self.kind, from_, to, self.revision, self.owner.uid, *args, **kwargs
        )

    def delete(self, *args, **kwargs):
        """Сокращение для::

        client.users_playlists_delete(self.kind, self.owner.uid)
        """
        return self.client.users_playlists_delete(self.kind, self.owner.uid, *args, **kwargs)

    async def delete_async(self, *args, **kwargs):
        """Сокращение для::

        await client.users_playlists_delete(self.kind, self.owner.uid)
        """
        return await self.client.users_playlists_delete(self.kind, self.owner.uid, *args, **kwargs)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['Playlist']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.Playlist`: Плейлист.
        """
        if not data:
            return None

        data = super(Playlist, cls).de_json(data, client)
        from yandex_music import (
            User,
            MadeFor,
            Cover,
            PlayCounter,
            TrackShort,
            PlaylistAbsence,
            Artist,
            TrackId,
            Contest,
            OpenGraphData,
            Brand,
        )

        data['owner'] = User.de_json(data.get('owner'), client)
        data['cover'] = Cover.de_json(data.get('cover'), client)
        data['cover_without_text'] = Cover.de_json(data.get('cover_without_text'), client)
        data['made_for'] = MadeFor.de_json(data.get('made_for'), client)
        data['tracks'] = TrackShort.de_list(data.get('tracks'), client)
        data['recent_tracks'] = TrackId.de_list(data.get('recent_tracks'), client)
        data['play_counter'] = PlayCounter.de_json(data.get('play_counter'), client)
        data['top_artist'] = Artist.de_list(data.get('top_artist'), client)
        data['contest'] = Contest.de_json(data.get('contest'), client)
        data['og_data'] = OpenGraphData.de_json(data.get('og_data'), client)
        data['dummy_cover'] = Cover.de_json(data.get('dummy_cover'), client)
        data['dummy_rollover_cover'] = Cover.de_json(data.get('dummy_rollover_cover'), client)
        data['branding'] = Brand.de_json(data.get('branding'), client)

        data['similar_playlists'] = Playlist.de_list(data.get('similar_playlists'), client)
        data['last_owner_playlists'] = Playlist.de_list(data.get('last_owner_playlists'), client)

        data['playlist_absence'] = PlaylistAbsence.de_json(data.get('playlist_absence'), client)  # на случай фикса
        if data.get('playlist_absense'):  # очепятка яндуха
            data['playlist_absence'] = PlaylistAbsence.de_json(data.get('playlist_absense'), client)
            data.pop('playlist_absense')

        return cls(client=client, **data)

    @classmethod
    def de_list(cls, data: dict, client: 'Client') -> List['Playlist']:
        """Десериализация списка объектов.

        Args:
            data (:obj:`list`): Список словарей с полями и значениями десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`list` из :obj:`yandex_music.Playlist`: Плейлисты.
        """
        if not data:
            return []

        return [cls.de_json(playlist, client) for playlist in data]

    # camelCase псевдонимы

    #: Псевдоним для :attr:`is_mine`
    isMine = is_mine
    #: Псевдоним для :attr:`playlist_id`
    playlistId = playlist_id
    #: Псевдоним для :attr:`get_recommendations`
    getRecommendations = get_recommendations
    #: Псевдоним для :attr:`get_recommendations_async`
    getRecommendationsAsync = get_recommendations_async
    #: Псевдоним для :attr:`download_animated_cover`
    downloadAnimatedCover = download_animated_cover
    #: Псевдоним для :attr:`download_animated_cover_async`
    downloadAnimatedCoverAsync = download_animated_cover_async
    #: Псевдоним для :attr:`download_og_image`
    downloadOgImage = download_og_image
    #: Псевдоним для :attr:`download_og_image_async`
    downloadOgImageAsync = download_og_image_async
    #: Псевдоним для :attr:`fetch_tracks`
    fetchTracks = fetch_tracks
    #: Псевдоним для :attr:`fetch_tracks_async`
    fetchTracksAsync = fetch_tracks_async
    #: Псевдоним для :attr:`insert_track`
    insertTrack = insert_track
    #: Псевдоним для :attr:`insert_track_async`
    insertTrackAsync = insert_track_async
    #: Псевдоним для :attr:`delete_tracks`
    deleteTracks = delete_tracks
    #: Псевдоним для :attr:`delete_tracks_async`
    deleteTracksAsync = delete_tracks_async
    #: Псевдоним для :attr:`rename_async`
    renameAsync = rename_async
    #: Псевдоним для :attr:`like_async`
    likeAsync = like_async
    #: Псевдоним для :attr:`dislike_async`
    dislikeAsync = dislike_async
    #: Псевдоним для :attr:`delete_async`
    deleteAsync = delete_async



from typing import TYPE_CHECKING, Optional

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client


@model
class Icon(YandexMusicObject):
    """Класс, представляющий иконку.

    Attributes:
        background_color (:obj:`str`): Цвет заднего фона в HEX.
        image_url (:obj:`str`): Ссылка на изображение.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    background_color: str
    image_url: str
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.background_color, self.image_url)

    def download(self, filename: str, size: str = '200x200') -> None:
        """Загрузка иконки.

        Args:
            filename (:obj:`str`): Путь для сохранения файла с названием и расширением.
            size (:obj:`str`, optional): Размер иконки.
        """
        self.client.request.download(self.get_url(size), filename)

    async def download_async(self, filename: str, size: str = '200x200') -> None:
        """Загрузка иконки.

        Args:
            filename (:obj:`str`): Путь для сохранения файла с названием и расширением.
            size (:obj:`str`, optional): Размер иконки.
        """
        await self.client.request.download(self.get_url(size), filename)

    def get_url(self, size: str = '200x200'):
        """Получение URL иконки.

        Args:
            size (:obj:`str`, optional): Размер иконки.
        """
        return f'https://{self.image_url.replace("%%", size)}'

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['Icon']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.Icon`: Иконка.
        """
        if not data:
            return None

        data = super(Icon, cls).de_json(data, client)

        return cls(client=client, **data)

    # camelCase псевдонимы

    #: Псевдоним для :attr:`download_async`
    downloadAsync = download_async



from typing import Any, TYPE_CHECKING, Optional, List, Union

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client, Cover, Ratings, Counts, Link, Track, Description, ArtistTracks, ArtistAlbums


@model
class Artist(YandexMusicObject):
    """Класс, представляющий исполнителя.

    Attributes:
        id (:obj:`int`): Уникальный идентификатор.
        error (:obj:`str`, optional): Сообщение об ошибке с объяснением почему не вернуло исполнителя.
        reason (:obj:`str`, optional): Причина отсутствия исполнителя (сообщение об ошибке).
        name (:obj:`str`, optional): Название.
        cover (:obj:`yandex_music.Cover`, optional): Обложка.
        various (:obj:`bool`, optional): TODO.
        composer (:obj:`bool`, optional): TODO.
        genres (:obj:`list` из :obj:`str`, optional): Жанры.
        og_image (:obj:`str`, optional): Ссылка на изображение для Open Graph.
        op_image (:obj:`str`, optional): Ссылка на изображение обложки. Используется когда не указано поле cover.
        no_pictures_from_search: TODO.
        counts (:obj:`yandex_music.Counts`, optional): Счётчики.
        available (:obj:`bool`, optional): Доступен ли для прослушивания.
        ratings (:obj:`yandex_music.Ratings`, optional): Рейтинги.
        links (:obj:`list` из :obj:`yandex_music.Link`, optional): Ссылки на ресурсы исполнителя.
        tickets_available (:obj:`bool`, optional): Имеются ли в продаже билеты на концерт.
        likes_count (:obj:`int`, optional): Количество лайков.
        popular_tracks (:obj:`list` :obj:`yandex_music.Track`, optional): Популярные треки.
        regions (:obj:`list` из :obj:`str`, optional): Регион TODO.
        decomposed (:obj:`list` из :obj:`str` и :obj:`yandex_music.Artist`, optional): Декомпозиция всех исполнителей.
            Лист, где чередуется разделитель и артист. Фиты и прочее.
        full_names: TODO.
        hand_made_description (:obj:`str`, optional): Описание от Яндекс TODO.
        description (:obj:`yandex_music.Description`, optional): Описание.
        countries (:obj:`list` из :obj:`str`, optional): Страны.
        en_wikipedia_link (:obj:`str`, optional): Адрес страницы на wikipedia.org.
        db_aliases (:obj:`list` из :obj:`str`, optional): Другие названия. Как правило названия на разных языках.
        aliases: TODO.
        init_date (:obj:`str`, optional): Дата начала в формате YYYY-MM-DD или YYYY.
        end_date (:obj:`str`, optional): Дата окончания в формате YYYY-MM-DD или YYYY.
        ya_money_id (:obj:`str`): Номер кошеляка Яндекс.Деньги TODO.
        client (:obj:`yandex_music.Client`): Клиент Yandex Music.
    """

    id: int
    error: Optional[str] = None
    reason: Optional[str] = None
    name: Optional[str] = None
    cover: Optional['Cover'] = None
    various: Optional[bool] = None
    composer: Optional[bool] = None
    genres: Optional[List[str]] = None
    og_image: Optional[str] = None
    op_image: Optional[str] = None
    no_pictures_from_search: Any = None
    counts: Optional['Counts'] = None
    available: Optional[bool] = None
    ratings: Optional['Ratings'] = None
    links: Optional[List['Link']] = None
    tickets_available: Optional[bool] = None
    likes_count: Optional[int] = None
    popular_tracks: Optional[List['Track']] = None
    regions: Optional[List[str]] = None
    decomposed: Optional[List[Union[str, 'Artist']]] = None
    full_names: Any = None
    hand_made_description: Optional[str] = None
    description: Optional['Description'] = None
    countries: Optional[List[str]] = None
    en_wikipedia_link: Optional[str] = None
    db_aliases: Optional[List[str]] = None
    aliases: Any = None
    init_date: Optional[str] = None
    end_date: Optional[str] = None
    ya_money_id: Optional[str] = None
    client: 'Client' = None

    def __post_init__(self):
        self._id_attrs = (self.id, self.name, self.cover)

    def download_og_image(self, filename: str, size: str = '200x200') -> None:
        """Загрузка изображения для Open Graph.

        Args:
            filename (:obj:`str`): Путь для сохранения файла с названием и расширением.
            size (:obj:`str`, optional): Размер обложки.
        """
        self.client.request.download(f'https://{self.og_image.replace("%%", size)}', filename)

    async def download_og_image_async(self, filename: str, size: str = '200x200') -> None:
        """Загрузка изображения для Open Graph.

        Args:
            filename (:obj:`str`): Путь для сохранения файла с названием и расширением.
            size (:obj:`str`, optional): Размер обложки.
        """
        await self.client.request.download(f'https://{self.og_image.replace("%%", size)}', filename)

    def download_op_image(self, filename: str, size: str = '200x200') -> None:
        """Загрузка обложки.

        Notes:
            Используйте это только когда нет self.cover!

        Args:
            filename (:obj:`str`): Путь для сохранения файла с названием и расширением.
            size (:obj:`str`, optional): Размер обложки.
        """
        self.client.request.download(f'https://{self.op_image.replace("%%", size)}', filename)

    async def download_op_image_async(self, filename: str, size: str = '200x200') -> None:
        """Загрузка обложки.

        Notes:
            Используйте это только когда нет self.cover!

        Args:
            filename (:obj:`str`): Путь для сохранения файла с названием и расширением.
            size (:obj:`str`, optional): Размер обложки.
        """
        await self.client.request.download(f'https://{self.op_image.replace("%%", size)}', filename)

    def like(self, *args, **kwargs) -> bool:
        """Сокращение для::

        client.users_likes_artists_add(artist.id, user.id *args, **kwargs)
        """
        return self.client.users_likes_artists_add(self.id, self.client.me.account.uid, *args, **kwargs)

    async def like_async(self, *args, **kwargs) -> bool:
        """Сокращение для::

        await client.users_likes_artists_add(artist.id, user.id *args, **kwargs)
        """
        return await self.client.users_likes_artists_add(self.id, self.client.me.account.uid, *args, **kwargs)

    def dislike(self, *args, **kwargs) -> bool:
        """Сокращение для::

        client.users_likes_artists_remove(artist.id, user.id *args, **kwargs)
        """
        return self.client.users_likes_artists_remove(self.id, self.client.me.account.uid, *args, **kwargs)

    async def dislike_async(self, *args, **kwargs) -> bool:
        """Сокращение для::

        await client.users_likes_artists_remove(artist.id, user.id *args, **kwargs)
        """
        return await self.client.users_likes_artists_remove(self.id, self.client.me.account.uid, *args, **kwargs)

    def get_tracks(self, page=0, page_size=20, *args, **kwargs) -> Optional['ArtistTracks']:
        """Сокращение для::

        client.artists_tracks(artist.id, page, page_size, *args, **kwargs)
        """
        return self.client.artists_tracks(self.id, page, page_size, *args, **kwargs)

    async def get_tracks_async(self, page=0, page_size=20, *args, **kwargs) -> Optional['ArtistTracks']:
        """Сокращение для::

        await client.artists_tracks(artist.id, page, page_size, *args, **kwargs)
        """
        return await self.client.artists_tracks(self.id, page, page_size, *args, **kwargs)

    def get_albums(self, page=0, page_size=20, sort_by='year', *args, **kwargs) -> Optional['ArtistAlbums']:
        """Сокращение для::

        client.artists_direct_albums(artist.id, page, page_size, sort_by, *args, **kwargs)
        """
        return self.client.artists_direct_albums(self.id, page, page_size, sort_by, *args, **kwargs)

    async def get_albums_async(self, page=0, page_size=20, sort_by='year', *args, **kwargs) -> Optional['ArtistAlbums']:
        """Сокращение для::

        await client.artists_direct_albums(artist.id, page, page_size, sort_by, *args, **kwargs)
        """
        return await self.client.artists_direct_albums(self.id, page, page_size, sort_by, *args, **kwargs)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['Artist']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.Artist`: Исполнитель.
        """
        if not data:
            return None

        data = super(Artist, cls).de_json(data, client)
        from yandex_music import Cover, Ratings, Counts, Link, Track, Description

        data['cover'] = Cover.de_json(data.get('cover'), client)
        data['ratings'] = Ratings.de_json(data.get('ratings'), client)
        data['counts'] = Counts.de_json(data.get('counts'), client)
        data['links'] = Link.de_list(data.get('links'), client)
        data['popular_tracks'] = Track.de_list(data.get('popular_tracks'), client)
        data['description'] = Description.de_json(data.get('description'), client)

        # Мне очень интересно увидеть как в яндухе на клиентах солвят свой бэковский костыль, пригласите на экскурсию
        if data.get('decomposed'):
            data['decomposed'] = [
                Artist.de_json(part, client) if isinstance(part, dict) else part for part in data['decomposed']
            ]

        return cls(client=client, **data)

    @classmethod
    def de_list(cls, data: dict, client: 'Client') -> List['Artist']:
        """Десериализация списка объектов.

        Args:
            data (:obj:`list`): Список словарей с полями и значениями десериализуемого объекта.
            client (:obj:`yandex_music.Client`): Клиент Yandex Music.

        Returns:
            :obj:`list` из :obj:`yandex_music.Artist`: Исполнители.
        """
        if not data:
            return []

        artists = list()
        for artist in data:
            artists.append(cls.de_json(artist, client))

        return artists

    # camelCase псевдонимы

    #: Псевдоним для :attr:`download_og_image`
    downloadOgImage = download_og_image
    #: Псевдоним для :attr:`download_og_image_async`
    downloadOgImageAsync = download_og_image_async
    #: Псевдоним для :attr:`download_op_image`
    downloadOpImage = download_op_image
    #: Псевдоним для :attr:`download_op_image_async`
    downloadOpImageAsync = download_op_image_async
    #: Псевдоним для :attr:`get_tracks`
    getTracks = get_tracks
    #: Псевдоним для :attr:`get_tracks_async`
    getTracksAsync = get_tracks_async
    #: Псевдоним для :attr:`get_albums`
    getAlbums = get_albums
    #: Псевдоним для :attr:`get_albums_async`
    getAlbumsAsync = get_albums_async
    #: Псевдоним для :attr:`like_async`
    likeAsync = like_async
    #: Псевдоним для :attr:`dislike_async`
    dislikeAsync = dislike_async



from typing import TYPE_CHECKING, Optional, List

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client, Playlist


@model
class GeneratedPlaylist(YandexMusicObject):
    """Класс, представляющий автоматически сгенерированный плейлист.

    Note:
        Известные значения `type`: `playlistOfTheDay`, `origin`, `recentTracks`, `neverHeard`, `podcasts`,
        `missedLikes`.

    Attributes:
        type (:obj:`str`): Тип сгенерированного плейлиста.
        ready (:obj:`bool`): Готовность плейлиста.
        notify (:obj:`bool`): Уведомлён ли пользователь об обновлении содержания.
        data (:obj:`yandex_music.Playlist`, optional): Сгенерированный плейлист.
        description (:obj:`list`, optional): Описание TODO.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    type: str
    ready: bool
    notify: bool
    data: Optional['Playlist']
    description: Optional[list] = None
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.type, self.ready, self.notify, self.data)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['GeneratedPlaylist']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.GeneratedPlaylist`: Автоматически сгенерированный плейлист.
        """
        if not data:
            return None

        data = super(GeneratedPlaylist, cls).de_json(data, client)
        from yandex_music import Playlist

        data['data'] = Playlist.de_json(data.get('data'), client)

        return cls(client=client, **data)

    @classmethod
    def de_list(cls, data: dict, client: 'Client') -> List['GeneratedPlaylist']:
        """Десериализация списка объектов.

        Args:
            data (:obj:`list`): Список словарей с полями и значениями десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`list` из :obj:`yandex_music.GeneratedPlaylist`: Автоматически сгенерированные плейлисты.
        """
        if not data:
            return []

        return [cls.de_json(generated_playlist, client) for generated_playlist in data]



from typing import TYPE_CHECKING, Optional, List

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client


@model
class Promotion(YandexMusicObject):
    """Класс, представляющий продвижение (рекламу).

    Note:
        В цвете может как оказаться HEX (`#6c65a9`), так и какой-нибудь `transparent`.

        Ссылка со схемой отличается от просто ссылки наличием `yandexmusic://` в начале.

    Attributes:
        promo_id (:obj:`str`): Уникальный идентификатор рекламы.
        title (:obj:`str`): Заголовок.
        subtitle (:obj:`str`): Подзаголовок.
        heading (:obj:`str`): Верхний заголовок.
        url (:obj:`str`): Ссылка.
        url_scheme (:obj:`str`): Ссылка с схемой.
        text_color (:obj:`str`): Цвет текста.
        gradient (:obj:`str`): Градиент TODO.
        image (:obj:`str`): Ссылка на рекламное изображение.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    promo_id: str
    title: str
    subtitle: str
    heading: str
    url: str
    url_scheme: str
    text_color: str
    gradient: str
    image: str
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (
            self.promo_id,
            self.title,
            self.subtitle,
            self.heading,
            self.url,
            self.url_scheme,
            self.text_color,
            self.gradient,
            self.image,
        )

    def download_image(self, filename: str, size: str = '300x300') -> None:
        """Загрузка рекламного изображения.

        Args:
            filename (:obj:`str`): Путь для сохранения файла с названием и расширением.
            size (:obj:`str`, optional): Размер изображения.
        """
        self.client.request.download(f'https://{self.image.replace("%%", size)}', filename)

    async def download_image_async(self, filename: str, size: str = '300x300') -> None:
        """Загрузка рекламного изображения.

        Args:
            filename (:obj:`str`): Путь для сохранения файла с названием и расширением.
            size (:obj:`str`, optional): Размер изображения.
        """
        await self.client.request.download(f'https://{self.image.replace("%%", size)}', filename)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['Promotion']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.Promotion`: Продвижение (реклама).
        """
        if not data:
            return None

        data = super(Promotion, cls).de_json(data, client)

        return cls(client=client, **data)

    @classmethod
    def de_list(cls, data: dict, client: 'Client') -> List['Promotion']:
        """Десериализация списка объектов.

        Args:
            data (:obj:`list`): Список словарей с полями и значениями десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`list` из :obj:`yandex_music.Promotion`: Продвижения (реклама).
        """
        if not data:
            return []

        promotions = list()
        for promotion in data:
            promotions.append(cls.de_json(promotion, client))

        return promotions

    # camelCase псевдонимы

    #: Псевдоним для :attr:`download_image`
    downloadImage = download_image
    #: Псевдоним для :attr:`download_image_async`
    downloadImageAsync = download_image_async



from typing import TYPE_CHECKING, Optional

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client


@model
class Images(YandexMusicObject):
    """Класс, представляющий изображение жанра.

    Attributes:
        _208x208 (:obj:`str`, optional): Ссылка на изображение размером 208 на 208.
        _300x300 (:obj:`str`, optional): Ссылка на изображение размером 300 на 300.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    _208x208: Optional[str] = None
    _300x300: Optional[str] = None
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self._208x208, self._300x300)

    def download_208x208(self, filename: str) -> None:
        """Загрузка изображения 208x208.

        Args:
            filename (:obj:`str`): Путь для сохранения файла с названием и расширением.
        """
        self.client.request.download(self._208x208, filename)

    def download_300x300(self, filename: str) -> None:
        """Загрузка изображения 300x300.

        Args:
            filename (:obj:`str`): Путь для сохранения файла с названием и расширением.
        """
        self.client.request.download(self._300x300, filename)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['Images']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.Images`: Изображение жанра.
        """
        if not data:
            return None

        data = super(Images, cls).de_json(data, client)

        return cls(client=client, **data)



from typing import List, TYPE_CHECKING, Optional

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client


@model
class Deactivation(YandexMusicObject):
    """Класс, представляющий способы деактивации мобильной услуги.

    Note:
        Известные значения поля `method`: `ussd`.

    Attributes:
        method (:obj:`str`): Метод отключения.
        instructions (:obj:`str`, optional): Инструкция.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    method: str
    instructions: Optional[str] = None
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.method, self.instructions)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['Deactivation']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.Deactivation`: Способ отключения услуги.
        """
        if not data:
            return None

        data = super(Deactivation, cls).de_json(data, client)

        return cls(client=client, **data)

    @classmethod
    def de_list(cls, data: dict, client: 'Client') -> List['Deactivation']:
        """Десериализация списка объектов.

        Args:
            data (:obj:`list`): Список словарей с полями и значениями десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`list` из :obj:`yandex_music.Deactivation`: Способы отключения услуги.
        """
        if not data:
            return []

        return [cls.de_json(deactivation, client) for deactivation in data]



from typing import TYPE_CHECKING, Optional

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client


@model
class PersonalPlaylistsData(YandexMusicObject):
    """Класс, представляющий дополнительную информацию о персональном плейлисте.

    Attributes:
        is_wizard_passed (:obj:`bool`): TODO.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    is_wizard_passed: bool
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.is_wizard_passed,)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['PersonalPlaylistsData']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.PersonalPlaylistsData`: Дополнительная информация о персональном плейлисте.
        """
        if not data:
            return None

        data = super(PersonalPlaylistsData, cls).de_json(data, client)

        return cls(client=client, **data)



from typing import TYPE_CHECKING, Optional, List

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client, Album, Track


@model
class AlbumEvent(YandexMusicObject):
    """Класс, представляющий альбом в событии фида.

    Attributes:
        album (:obj:`yandex_music.Album` | :obj:`None`): Альбом.
        tracks (:obj:`list` из :obj:`yandex_music.Track`): Треки.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    album: Optional['Album']
    tracks: List['Track']
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.album, self.tracks)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['AlbumEvent']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.AlbumEvent`: Альбом в событии фида.
        """
        if not data:
            return None

        data = super(AlbumEvent, cls).de_json(data, client)
        from yandex_music import Album, Track

        data['album'] = Album.de_json(data.get('album'), client)
        data['tracks'] = Track.de_list(data.get('tracks'), client)

        return cls(client=client, **data)

    @classmethod
    def de_list(cls, data: dict, client: 'Client') -> List['AlbumEvent']:
        """Десериализация списка объектов.

        Args:
            data (:obj:`list`): Список словарей с полями и значениями десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`list` из :obj:`yandex_music.AlbumEvent`: Альбомы в событии фида.
        """
        if not data:
            return []

        album_events = list()
        for album_event in data:
            album_events.append(cls.de_json(album_event, client))

        return album_events



from typing import List, TYPE_CHECKING, Optional

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client, Deactivation


@model
class Operator(YandexMusicObject):
    """Класс, представляющий услугу сотового оператора.

    Attributes:
        product_id (:obj:`str`): Уникальный идентификатор продукта сервиса Яндекс.Музыка.
        phone (:obj:`str`): Мобильный номер, на который подключена услуга.
        payment_regularity (:obj:`str`): Регулярность оплаты.
        deactivation (:obj:`list` из :obj:`yandex_music.Deactivation`): Способы деактивации.
        title (:obj:`str`): Название услуги.
        suspended (:obj:`bool`): Приостановлено.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    product_id: str
    phone: str
    payment_regularity: str
    deactivation: List['Deactivation']
    title: str
    suspended: bool
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.product_id, self.phone)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['Operator']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.Operator`: Услуга сотового оператора.
        """
        if not data:
            return None

        from yandex_music import Deactivation

        data['deactivation'] = Deactivation.de_list(data.get('deactivation'), client)

        data = super(Operator, cls).de_json(data, client)

        return cls(client=client, **data)

    @classmethod
    def de_list(cls, data: dict, client: 'Client') -> List['Operator']:
        """Десериализация списка объектов.

        Args:
            data (:obj:`list`): Список словарей с полями и значениями десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`list` из :obj:`yandex_music.Operator`: Услуги сотового оператора.
        """
        if not data:
            return []

        return [cls.de_json(operator, client) for operator in data]



from typing import TYPE_CHECKING, Optional, List, Iterator

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client, Track


@model
class SimilarTracks(YandexMusicObject):
    """Класс, представляющий список похожих треков на другой трек.

    Attributes:
        track (:obj:`yandex_music.Track`): Трек.
        similar_tracks (:obj:`list` из :obj:`yandex_music.Track`): Похожие треки на `track`.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    track: Optional['Track']
    similar_tracks: List['Track']
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.track, self.similar_tracks)

    def __getitem__(self, item) -> 'Track':
        return self.similar_tracks[item]

    def __iter__(self) -> Iterator['Track']:
        return iter(self.similar_tracks)

    def __len__(self) -> int:
        return len(self.similar_tracks)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['SimilarTracks']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.SimilarTracks`: Список похожих треков на другой трек.
        """
        if not data:
            return None

        data = super(SimilarTracks, cls).de_json(data, client)
        from yandex_music import Track

        data['track'] = Track.de_json(data.get('track'), client)
        data['similar_tracks'] = Track.de_list(data.get('similar_tracks'), client)

        return cls(client=client, **data)



from typing import TYPE_CHECKING, Optional, List

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client


@model
class Link(YandexMusicObject):
    """Класс, представляющий ссылку на официальную страницу исполнителя.

    Note:
        Известные типы страниц: `official` - официальный сайт и `social` - социальная сеть.

    Attributes:
        title (:obj:`str`): Название страницы.
        href (:obj:`str`): URL страницы.
        type (:obj:`str`): Тип страницы.
        social_network (:obj:`str`, optional): Название социальной сети.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    title: str
    href: str
    type: str
    social_network: Optional[str] = None
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.title, self.href, self.type)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['Link']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.Link`: Ссылка на официальную страницу исполнителя.
        """
        if not data:
            return None

        data = super(Link, cls).de_json(data, client)

        return cls(client=client, **data)

    @classmethod
    def de_list(cls, data: dict, client: 'Client') -> List['Link']:
        """Десериализация списка объектов.

        Args:
            data (:obj:`list`): Список словарей с полями и значениями десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`list` из :obj:`yandex_music.Link`: Ссылки на официальные страницы исполнителя.
        """
        if not data:
            return []

        links = list()
        for link in data:
            links.append(cls.de_json(link, client))

        return links



from typing import TYPE_CHECKING, Optional

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client


@model
class Counts(YandexMusicObject):
    """Класс, представляющий счётчик некоторых значений исполнителя.

    Note:
        Под дополнительными подразумеваются треки и альбомы, в которых артист задействован, но не является его автором.
        Так же в дополнительные альбомы входят сборники.

    Attributes:
        tracks (:obj:`int`): Количество треков.
        direct_albums (:obj:`int`): Количество альбомов.
        also_albums (:obj:`int`): Количество дополнительных треков.
        also_tracks (:obj:`int`): Количество дополнительных альбомов.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    tracks: int
    direct_albums: int
    also_albums: int
    also_tracks: int
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.tracks, self.direct_albums, self.also_albums, self.also_tracks)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['Counts']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.Counts`: Cчётчик некоторых значений исполнителя.
        """
        if not data:
            return None

        data = super(Counts, cls).de_json(data, client)

        return cls(client=client, **data)



from typing import TYPE_CHECKING, Optional, List

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client, ShotData


@model
class Shot(YandexMusicObject):
    """Класс, представляющий шот от Алисы.

    Note:
        Известные значения поля `status`: `ready`.

    Attributes:
        order (:obj:`int`): Порядковый номер при воспроизведении.
        played (:obj:`bool`): Был ли проигран шот.
        shot_data (:obj:`yandex_music.ShotData`): Основная информация о шоте.
        shot_id (:obj:`str`): Уникальный идентификатор шота.
        status (:obj:`str`): Статус шота.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    order: int
    played: bool
    shot_data: 'ShotData'
    shot_id: str
    status: str
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.order, self.played, self.shot_data, self.shot_id, self.status)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['Shot']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.Shot`: Шот от Алисы.
        """
        if not data:
            return None

        data = super(Shot, cls).de_json(data, client)
        from yandex_music import ShotData

        data['shot_data'] = ShotData.de_json(data.get('shot_data'), client)

        return cls(client=client, **data)

    @classmethod
    def de_list(cls, data: dict, client: 'Client') -> List['Shot']:
        """Десериализация списка объектов.

        Args:
            data (:obj:`list`): Список словарей с полями и значениями десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`list` из :obj:`yandex_music.Shot`: Шоты от Алисы.
        """
        if not data:
            return []

        shots = list()
        for shot in data:
            shots.append(cls.de_json(shot, client))

        return shots



from typing import TYPE_CHECKING, Optional, List

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client, Product, Price


@model
class Settings(YandexMusicObject):
    """Класс, представляющий предложения по покупке.

    Attributes:
        in_app_products (:obj:`list` из :obj:`yandex_music.Product`): Продаваемые продукты внутри приложения.
        native_products (:obj:`list`) из :obj:`yandex_music.Product`: Продаваемые продукты всплывающими окнами.
        web_payment_url (:obj:`str`): Ссылка для осуществления платежа.
        web_payment_month_product_price (:obj:`yandex_music.Price`, optional): Цена продукта за месяц.
        promo_codes_enabled (:obj:`bool`): Доступно ли использование промо-кодов.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    in_app_products: List['Product']
    native_products: List['Product']
    web_payment_url: str
    promo_codes_enabled: bool
    web_payment_month_product_price: Optional['Price'] = None
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.in_app_products, self.native_products, self.web_payment_url, self.promo_codes_enabled)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['Settings']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.Settings`: Предложение по покупке.
        """
        if not data:
            return None

        data = super(Settings, cls).de_json(data, client)
        from yandex_music import Product, Price

        data['in_app_products'] = Product.de_list(data.get('in_app_products'), client)
        data['native_products'] = Product.de_list(data.get('native_products'), client)
        data['web_payment_month_product_price'] = Price.de_json(data.get('web_payment_month_product_price'), client)

        return cls(client=client, **data)



from typing import TYPE_CHECKING, Optional

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client


@model
class Lyrics(YandexMusicObject):
    """Класс, представляющий текст трека.

    Attributes:
        id (:obj:`int`): Уникальный идентификатор текста трека.
        lyrics (:obj:`str`): Первые строки текст песни.
        has_rights (:obj:`bool`): Есть ли права.
        full_lyrics (:obj:`str`): Текст песни.
        show_translation (:obj:`bool`): Доступен ли перевод.
        text_language (:obj:`str`, optional): Язык песни.
        url (:obj:`str`, optional): Ссылка на источник перевода. Обычно genius.com.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
        **kwargs: Произвольные ключевые аргументы полученные от API.
    """

    id: int
    lyrics: str
    full_lyrics: str
    has_rights: bool
    show_translation: bool
    text_language: Optional[str] = None
    url: Optional[str] = None
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (
            self.id,
            self.lyrics,
            self.full_lyrics,
            self.has_rights,
            self.text_language,
            self.show_translation,
        )

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['Lyrics']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.Lyrics`: Текст трека.
        """
        if not data:
            return None

        data = super(Lyrics, cls).de_json(data, client)

        return cls(client=client, **data)



from typing import TYPE_CHECKING, Optional, List

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client, Id, Sequence


@model
class StationTracksResult(YandexMusicObject):
    """Класс, представляющий последовательность треков станции.

    Attributes:
        id (:obj:`yandex_music.Id`): Уникальный идентификатор станции.
        sequence (:obj:`list` из :obj:`yandex_music.Sequence`): Последовательность треков.
        batch_id (:obj:`str`): Уникальный идентификатор партии (последовательности).
        pumpkin (:obj:`bool`): Хэллоуин.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    id: Optional['Id']
    sequence: List['Sequence']
    batch_id: str
    pumpkin: bool
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.id, self.sequence, self.batch_id, self.pumpkin)

    @classmethod
    def de_json(cls, data, client) -> Optional['StationTracksResult']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.StationTracksResult`: Последовательность треков станции.
        """
        if not data:
            return None

        data = super(StationTracksResult, cls).de_json(data, client)
        from yandex_music import Id, Sequence

        data['id'] = Id.de_json(data.get('id'), client)
        data['sequence'] = Sequence.de_list(data.get('sequence'), client)

        return cls(client=client, **data)



from typing import TYPE_CHECKING, Optional, List

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client, PassportPhone


@model
class Account(YandexMusicObject):
    """Класс, представляющий основную информацию об аккаунте пользователя.

    Attributes:
        now (:obj:`str`): Текущая дата и время.
        uid (:obj:`int`, optional): Уникальный идентификатор.
        login (:obj:`str`, optional): Виртуальное имя (обычно e-mail).
        full_name (:obj:`str`, optional): Полное имя (имя и фамилия).
        second_name (:obj:`str`, optional): Фамилия.
        first_name (:obj:`str`, optional): Имя.
        display_name (:obj:`str`, optional): Отображаемое имя.
        service_available (:obj:`bool`): Доступен ли сервис.
        hosted_user (:obj:`bool`, optional): Является ли пользователем чьим-то другим.
        birthday (:obj:`str`, optional): Дата рождения.
        region (:obj:`int`, optional): Регион.
        passport_phones (:obj:`list` из :obj:`yandex_music.PassportPhone`): Мобильные номера.
        registered_at (:obj:`str`, optional): Дата создания учётной записи.
        has_info_for_app_metrica (:obj:`bool`, optional): Наличие информации для App Metrica.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
        **kwargs: Произвольные ключевые аргументы полученные от API.
    """

    now: str
    service_available: bool
    region: Optional[int] = None
    uid: Optional[int] = None
    login: Optional[str] = None
    full_name: Optional[str] = None
    second_name: Optional[str] = None
    first_name: Optional[str] = None
    display_name: Optional[str] = None
    hosted_user: Optional[bool] = None
    birthday: Optional[str] = None
    passport_phones: List['PassportPhone'] = None
    registered_at: Optional[str] = None
    has_info_for_app_metrica: bool = False
    client: Optional['Client'] = None

    def __post_init__(self):
        if self.uid:
            self._id_attrs = (self.uid,)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['Account']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.Account`: Основная информация об аккаунте пользователя.
        """
        if not data:
            return None

        data = super(Account, cls).de_json(data, client)
        from yandex_music import PassportPhone

        data['passport_phones'] = PassportPhone.de_list(data.get('passport_phones'), client)

        return cls(client=client, **data)



from typing import TYPE_CHECKING, Optional, List

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client


@model
class LicenceTextPart(YandexMusicObject):
    """Класс, представляющий часть текста с ссылкой на лицензионное соглашение.

    Attributes:
        text (:obj:`str`): Часть текста (строка).
        url (:obj:`str`, optional): Ссылка на лицензионное соглашение.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    text: str
    url: Optional[str] = None
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.text,)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['PassportPhone']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.LicenceTextPart`: Строка лицензионного соглашения.
        """
        if not data:
            return None

        data = super(LicenceTextPart, cls).de_json(data, client)

        return cls(client=client, **data)

    @classmethod
    def de_list(cls, data: dict, client: 'Client') -> List['LicenceTextPart']:
        """Десериализация списка объектов.

        Args:
            data (:obj:`list`): Список словарей с полями и значениями десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`list` из :obj:`yandex_music.LicenceTextPart`: Строки текста с ссылкой на лицензионное соглашение.
        """
        if not data:
            return []

        return [cls.de_json(row, client) for row in data]



from typing import TYPE_CHECKING, Optional, Union

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client


@model
class AdParams(YandexMusicObject):
    """Класс, представляющий параметры рекламного объявления.

    Note:
        Известные дополнительные параметры(`other_params`): `user:{ID}`.

    Attributes:
        partner_id (:obj:`str` | :obj:`int`): Уникальный идентификатор заказчика рекламы.
        category_id (:obj:`str` | :obj:`int`): Уникальный идентификатор категории рекламы.
        page_ref (:obj:`str`): Ссылка на ссылающуюся страницу.
        target_ref (:obj:`str`): Ссылка на целевую страницу.
        other_params (:obj:`str`): Другие параметры.
        ad_volume (:obj:`int`): Громкость воспроизводимой рекламы.
        genre_id (:obj:`str`, optional): Уникальный идентификатор жанра.
        genre_name (:obj:`str`, optional): Название жанра.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    partner_id: Union[str, int]
    category_id: Union[str, int]
    page_ref: str
    target_ref: str
    other_params: str
    ad_volume: int
    genre_id: Optional[str] = None
    genre_name: Optional[str] = None
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (
            self.partner_id,
            self.category_id,
            self.page_ref,
            self.target_ref,
            self.other_params,
            self.ad_volume,
        )

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['AdParams']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.AdParams`: Параметры рекламного объявления.
        """
        if not data:
            return None

        data = super(AdParams, cls).de_json(data, client)

        return cls(client=client, **data)



from typing import TYPE_CHECKING, Optional, List

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client


@model
class VideoSupplement(YandexMusicObject):
    """Класс, представляющий видеоклипы.

    Attributes:
        cover (:obj:`str`): URL на обложку видео.
        provider (:obj:`str`): Сервис поставляющий видео.
        title (:obj:`str`, optional): Название видео.
        provider_video_id (:obj:`str`, optional): Уникальный идентификатор видео на сервисе.
        url (:obj:`str`, optional): URL на видео.
        embed_url (:obj:`str`, optional): URL на видео, находящегося на серверах Яндекса.
        embed (:obj:`str`, optional): HTML тег для встраивания видео.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    cover: str
    provider: str
    title: Optional[str] = None
    provider_video_id: Optional[str] = None
    url: Optional[str] = None
    embed_url: Optional[str] = None
    embed: Optional[str] = None
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.cover, self.title, self.provider_video_id)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['VideoSupplement']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.VideoSupplement`: Видеоклип.
        """
        if not data:
            return None

        data = super(VideoSupplement, cls).de_json(data, client)

        return cls(client=client, **data)

    @classmethod
    def de_list(cls, data: dict, client: 'Client') -> List['VideoSupplement']:
        """Десериализация списка объектов.

        Args:
            data (:obj:`list`): Список словарей с полями и значениями десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`list` из :obj:`yandex_music.VideoSupplement`: Видеоклипы.
        """
        if not data:
            return []

        videos = list()
        for video in data:
            videos.append(cls.de_json(video, client))

        return videos



from typing import TYPE_CHECKING, Optional

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client


@model
class RenewableRemainder(YandexMusicObject):
    """Класс, представляющий напоминания о продлении подписки.

    Note:
        Присутствует только тогда, когда автопродление отключено.

    Attributes:
        days (:obj:`int`): Количество дней (до окончания подписки, по всей видимости).
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    days: int
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.days,)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['RenewableRemainder']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.PassportPhone`: Напоминание о продлении подписки.
        """
        if not data:
            return None

        data = super(RenewableRemainder, cls).de_json(data, client)

        return cls(client=client, **data)



from typing import TYPE_CHECKING, Optional

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client


@model
class Context(YandexMusicObject):
    """Класс, представляющий содержимое очереди.

    Note:
        Известные значения поля `type`: `various`, `my_music`, `radio`, `playlist`, `artist`.

        Тип `various` используется при прослушивании из раздела "Моя музыка" с сайта, а `my_music` с мобильных клиентов.

        Поле `description` зачастую есть только когда `type` имеет значение `my_music` или `various`.

        При `type` равным `my_music` или `various` поле `id` отсутствует.

    Attributes:
        type (:obj:`str`): Тип содержимого (по чём построена очередь).
        id (:obj:`str`, optional): Уникальный идентификатор типа содержимого (плейлиста, альбома и т.д.).
        description (:obj:`str`, optional): Описание содержимого (например, название плейлиста, радиостанции).
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    type: str
    id: Optional[str] = None
    description: Optional[str] = None
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.type, self.id)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['Context']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.Context`: Содержимое очереди.
        """
        if not data:
            return None

        data = super(Context, cls).de_json(data, client)

        return cls(client=client, **data)



from typing import TYPE_CHECKING, Optional

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client


@model
class CaseForms(YandexMusicObject):
    """Класс, представляющий склонение имени.

    Attributes:
        nominative (:obj:`str`): Именительный.
        genitive (:obj:`str`): Родительный.
        dative (:obj:`str`): Дательный.
        accusative (:obj:`str`): Винительный.
        instrumental (:obj:`str`): Творительный.
        prepositional (:obj:`str`): Предложный.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    nominative: str
    genitive: str
    dative: str
    accusative: str
    instrumental: str
    prepositional: str
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (
            self.nominative,
            self.genitive,
            self.dative,
            self.accusative,
            self.instrumental,
            self.prepositional,
        )

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['CaseForms']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.CaseForms`: TODO.
        """
        if not data:
            return None

        data = super(CaseForms, cls).de_json(data, client)

        return cls(client=client, **data)



from typing import TYPE_CHECKING, Optional, List

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client, StationResult


@model
class Dashboard(YandexMusicObject):
    """Класс, представляющий рекомендованные станций пользователя.

    Attributes:
        dashboard_id (:obj:`str`): Уникальный идентификатор панели.
        stations (:obj:`list` из :obj:`yandex_music.StationResult`): Станции со всеми возможными настройками и
            параметрами.
        pumpkin (:obj:`bool`): Хэллоуин.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    dashboard_id: str
    stations: List['StationResult']
    pumpkin: bool
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.dashboard_id, self.stations, self.pumpkin)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['Dashboard']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.Dashboard`: Рекомендованные станций пользователя.
        """
        if not data:
            return None

        data = super(Dashboard, cls).de_json(data, client)
        from yandex_music import StationResult

        data['stations'] = StationResult.de_list(data.get('stations'), client)

        return cls(client=client, **data)



from typing import TYPE_CHECKING, Optional, List

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client, Product, User


@model
class AutoRenewable(YandexMusicObject):
    """Класс, представляющий информацию об автопродлении подписки.

    Attributes:
        expires (:obj:`str`): Дата истечения подписки.
        vendor (:obj:`str`): Продавец.
        vendor_help_url (:obj:`str`): Ссылка на страницу помощи продавца.
        product_id (:obj:`str`): Уникальный идентификатор продукта.
        finished (:obj:`bool`): Завершенность автопродления.
        master_info (:obj:`yandex_music.User`, optional): Главный в семейной подписке.
        product (:obj:`yandex_music.Product`, optional): Продукт.
        order_id (:obj:`int`): Уникальный идентификатор заказа.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    expires: str
    vendor: str
    vendor_help_url: str
    product: Optional['Product']
    finished: bool
    master_info: Optional['User'] = None
    product_id: Optional[str] = None
    order_id: Optional[int] = None
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.expires, self.vendor, self.vendor_help_url, self.product, self.finished)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['AutoRenewable']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.AutoRenewable`: Информация об автопродлении подписки.
        """
        if not data:
            return None

        data = super(AutoRenewable, cls).de_json(data, client)
        from yandex_music import Product, User

        data['product'] = Product.de_json(data.get('product'), client)
        data['master_info'] = User.de_json(data.get('master_info'), client)

        return cls(client=client, **data)

    @classmethod
    def de_list(cls, data: dict, client: 'Client') -> List['AutoRenewable']:
        """Десериализация списка объектов.

        Args:
            data (:obj:`list`): Список словарей с полями и значениями десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`list` из :obj:`yandex_music.AutoRenewable`: Информация об автопродлении подписки.
        """
        if not data:
            return []

        return [cls.de_json(auto_renewable, client) for auto_renewable in data]



from typing import TYPE_CHECKING, Optional, List, Union

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client


@model
class Video(YandexMusicObject):
    """Класс, представляющий видео.

    Attributes:
         title (:obj:`str`): Название видео.
         cover (:obj:`str`, optional): Ссылка на изображение.
         embed_url (:obj:`str`, optional): Ссылка на видео.
         provider (:obj:`str`, optional): Провайдер видео.
         provider_video_id (:obj:`int` | :obj:`str`, optional): Идентификатор видео.
         youtube_url (:obj:`str`, optional): Ссылка на видео Youtube.
         thumbnail_url (:obj:`str`, optional): Ссылка на изображение.
         duration (:obj:`int`, optional): Длительность видео в секундах.
         text (:obj:`str`, optional): Текст.
         html_auto_play_video_player (:obj:`str`, optional): HTML тег для встраивания в разметку страницы.
         regions (:obj:`list` из :obj:`str`, optional): Регион TODO.
         client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    title: str
    cover: Optional[str] = None
    embed_url: Optional[str] = None
    provider: Optional[str] = None
    provider_video_id: Optional[Union[int, str]] = None
    youtube_url: Optional[str] = None
    thumbnail_url: Optional[str] = None
    duration: Optional[int] = None
    text: Optional[str] = None
    html_auto_play_video_player: Optional[str] = None
    regions: Optional[List[str]] = None
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.provider_video_id, self.youtube_url, self.title)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['Video']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.Video`: Видео.
        """
        if not data:
            return None

        data = super(Video, cls).de_json(data, client)

        return cls(client=client, **data)

    @classmethod
    def de_list(cls, data: dict, client: 'Client') -> List['Video']:
        """Десериализация списка объектов.

        Args:
            data (:obj:`list`): Список словарей с полями и значениями десериализуемого объекта.
            client (:obj:`yandex_music.Client`): Клиент Yandex Music.

        Returns:
            :obj:`list` из :obj:`yandex_music.Video`: Видео.
        """
        if not data:
            return []

        videos = list()
        for video in data:
            videos.append(cls.de_json(video, client))

        return videos



from typing import TYPE_CHECKING, Optional, List

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client, TrackShortOld


@model
class PlayContext(YandexMusicObject):
    """Класс, представляющий проигрываемый контекст.

    Note:
        Известные значения поля `client_`: `android`.

        Поле `context` хранит в себе место воспроизведения, например, `playlist`.

        Поле `context_item` хранит в себе уникальный идентификатор context'a, т.е. в нашем случае playlist'a.

    Attributes:
        client_ (:obj:`str`): Клиент.
        context (:obj:`str`): Тип контекста.
        context_item (:obj:`str`): Предмет контекста.
        tracks (:obj:`list` из :obj:`yandex_music.TrackShortOld`): Треки.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    client_: str
    context: str
    context_item: str
    tracks: List['TrackShortOld']
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.client_, self.context_item, self.context_item, self.tracks)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['PlayContext']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.PlayContext`: Проигрываемый контекст.
        """
        if not data:
            return None

        data = super(PlayContext, cls).de_json(data, client)
        from yandex_music import TrackShortOld

        data['tracks'] = TrackShortOld.de_list(data.get('tracks'), client)

        return cls(client=client, **data)



from typing import TYPE_CHECKING, Optional, List

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client


@model
class Brand(YandexMusicObject):
    """Класс, представляющий бренд плейлиста.

    Note:
        Отслеживание просмотров на сторонник сервисах бренда, рекомендация следующего контента.

    Attributes:
        image (:obj:`str`): Ссылка на изображение.
        background (:obj:`str`): Цвет заднего фона.
        reference (:obj:`str`): URI ссылка на содержимое.
        pixels (:obj:`list` из :obj:`str`): Ссылки на gif изображения для отслеживания просмотров (web beacon).
        theme (:obj:`str`): Тема оформления.
        playlist_theme (:obj:`str`): Тема плейлиста TODO.
        button (:obj:`str`): Текст кнопки.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    image: str
    background: str
    reference: str
    pixels: List[str]
    theme: str
    playlist_theme: str
    button: str
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.image, self.reference, self.pixels)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['Brand']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.Brand`: Бренд плейлиста.
        """
        if not data:
            return None

        data = super(Brand, cls).de_json(data, client)

        return cls(client=client, **data)



from typing import TYPE_CHECKING, Optional

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client


@model
class PlayCounter(YandexMusicObject):
    """Класс, представляющий счётчик дней.

    Note:
        Присутствует только у плейлиста дня. Счётчик считает количество дней подряд, на протяжении которых был
        прослушан плейлист.

    Attributes:
        value (:obj:`int`): Значение (количество дней).
        description (:obj:`str`): Описание счётчика.
        updated (:obj:`bool`): Обновлён ли сегодня (в этих сутках).
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    value: int
    description: str
    updated: bool
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.value, self.description, self.updated)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['PlayCounter']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.PlayCounter`: Счетчик дней.
        """
        if not data:
            return None

        data = super(PlayCounter, cls).de_json(data, client)

        return cls(client=client, **data)



from typing import TYPE_CHECKING, Optional

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client


@model
class Plus(YandexMusicObject):
    """Класс, представляющий `Plus` подписку.

    Attributes:
        has_plus (:obj:`bool`): Наличие.
        is_tutorial_completed (:obj:`bool`): Закончено ли руководство.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    has_plus: bool
    is_tutorial_completed: bool
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.has_plus, self.is_tutorial_completed)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['Plus']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.Plus`: Plus подписка.
        """
        if not data:
            return None

        data = super(Plus, cls).de_json(data, client)

        return cls(client=client, **data)



from typing import TYPE_CHECKING, Optional, List, Union

from yandex_music import YandexMusicObject, Artist, Album, Track, Playlist, Video, User
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client


de_json_result = {
    'track': Track.de_list,
    'artist': Artist.de_list,
    'album': Album.de_list,
    'playlist': Playlist.de_list,
    'video': Video.de_list,
    'user': User.de_list,
    'podcast': Album.de_list,
    'podcast_episode': Track.de_list,
}


@model
class SearchResult(YandexMusicObject):
    """Класс, представляющий результаты поиска.

    Note:
        Значения поля `type`: `track`, `artist`, `playlist`, `album`, `video`.

    Attributes:
        type (:obj:`str`):  Тип результата.
        total (:obj:`int`): Количество результатов.
        per_page (:obj:`int`): Максимальное количество результатов на странице.
        order (:obj:`int`): Позиция блока.
        results (:obj:`list` из :obj:`yandex_music.Track` | :obj:`yandex_music.Artist` | :obj:`yandex_music.Album` \
            | :obj:`yandex_music.Playlist` | :obj:`yandex_music.Video`): Результаты поиска.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    type: str
    total: int
    per_page: int
    order: int
    results: List[Union[Track, Artist, Album, Playlist, Video]]
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.total, self.per_page, self.order, self.results)

    @classmethod
    def de_json(cls, data: dict, client: 'Client', type_: str = None) -> Optional['SearchResult']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            type_ (:obj:`str`, optional): Тип результата.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.SearchResult`: Результаты поиска.
        """
        if not data:
            return None

        data = super(SearchResult, cls).de_json(data, client)
        data['type'] = type_
        data['results'] = de_json_result.get(type_)(data.get('results'), client)

        return cls(client=client, **data)
